;;-----------------------------LICENSE NOTICE------------------------------------
;;  This file is part of CPCtelera: An Amstrad CPC Game Engine 
;;  Copyright (C) 2018 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
;;
;;  This program is free software: you can redistribute it and/or modify
;;  it under the terms of the GNU Lesser General Public License as published by
;;  the Free Software Foundation, either version 3 of the License, or
;;  (at your option) any later version.
;;
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU Lesser General Public License for more details.
;;
;;  You should have received a copy of the GNU Lesser General Public License
;;  along with this program.  If not, see <http://www.gnu.org/licenses/>.
;;-------------------------------------------------------------------------------

.module main

;; Include all CPCtelera constant definitions, macros and variables
.include "cpctelera.h.s"
.include "man/entity.h.s"
.include "man/game.h.s"
.include "sys/render.h.s"
.include "sys/physics.h.s"
.include "sys/collision.h.s"
.include "man/menu.h.s"
.include "man/menu1.h.s"
.include "man/menu2.h.s"

.area _DATA
.area _CODE

estado_partida:: .db 1
estado_partida_ant:: .db 1


;; 
;; Declare all function entry points as global symbols for the compiler.
;; (The linker will know what to do with them)
;; WARNING: Every global symbol declared will be linked, so DO NOT declare 
;; symbols for functions you do not use.
;;
.globl cpct_getScreenPtr_asm
.globl cpct_drawSolidBox_asm
.globl cpct_setVideoMode_asm
.globl cpct_setPalette_asm
.globl cpct_drawSprite_asm

.globl cpct_disableFirmware_asm        ;;Antes no estaba. La he puesto por él la tenía


decompress_buffer = 0x454F       ;;Lugar donde hacemos la descompresion (niveles + tileset)

levels_maxsize = 0x1F4  ;;500 en dec

tiles_maxsize = 0x300   ;;768 en dec
decompress_buffer_end_level == decompress_buffer + levels_maxsize - 1

decompress_buffer_end_tiles == decompress_buffer_end_level + tiles_maxsize

tileset_ptr == decompress_buffer_end_level + 1
mapa_ptr == decompress_buffer + 0

.globl _my_fenyx_song
.globl _my_demo_song

.globl cpct_akp_musicInit_asm

fin_menu: .db 00
contador: .db 06

int_handler::

   push af
   push bc
   push de
   push hl

   ld    a, (contador)
   dec   a
   jr    nz, _cont

   _zero:
   ld a, (fin_menu)
   cp #0
   jp nz, _saltito

   call  cpct_akp_musicPlay_asm
   call  cpct_scanKeyboard_if_asm

   _saltito:

   ld a, (fin_menu)
   cp #0
   jr z, _seguir


   ld a, (fin_menu)
   cp #1
   jr nz, _seguir
   call cpct_akp_stop_asm
   ld a, #3
   ld (fin_menu), a


   _seguir:
   ld    a, #12

   _cont:
   ld    (contador), a
   ;;ld       h, a
   ;;ld       l, #16
;;
   ;;call     cpct_setPALColour_asm


   pop hl
   pop de
   pop bc
   pop af

   ei

   reti

set_int_handler:

   call cpct_waitVSYNC_asm
   halt
   halt
   call cpct_waitVSYNC_asm

   ld    hl, #0x38
   ld    (hl), #0xC3
   inc   hl
   ld    (hl), #<int_handler
   inc   hl
   ld    (hl), #>int_handler
   inc   hl
   ld    (hl), #0xC9

   ret
;;;;;;;;;;;;;;;;;;;;;;;;;

;;
;; MAIN function. This is the entry point of the application.
;;    _main:: global symbol is required for correctly compiling and linking
;;


;;  IMPORTANTE
;;  #_hola      -> Hacemos referencia a la direccion de memoria
;;  (_hola)     -> Hacemos referencia al contenido almacenado en esa direccion de memoria

_main::

   call set_int_handler
   

   ld de, #_my_fenyx_song
   call cpct_akp_musicInit_asm


   ld a, #01
   ld (estado_partida), a
   ld (estado_partida_ant), a

   ;;call     cpct_disableFirmware_asm


   ld       hl, #_my_tiles_pack_end
   ld       de, #decompress_buffer_end_tiles
   call     cpct_zx7b_decrunch_s_asm

   ;;NO DESCOMENTAR 
   ;;ld       hl, #_my_levels_pack_end
   ;;ld       de, #decompress_buffer_end
   ;;call     cpct_zx7b_decrunch_s_asm


   call     man_menu_init
   call     man_menu_run

   new_game:
   ld a, #01
   ld (estado_partida), a
   ld (estado_partida_ant), a

   ;;call     cpct_disableFirmware_asm


   ld       hl, #_my_tiles_pack_end
   ld       de, #decompress_buffer_end_tiles
   call     cpct_zx7b_decrunch_s_asm

   call     man_menu1_init
   call     man_menu1_run
   call     man_menu2_init
   call     man_menu2_run
   
   ld a, #1
   ld (fin_menu), a
   back:
   
   

   call     man_game_init
   call     man_hud_init
   call     man_game_play

   ld       a, (estado_partida)
   cp       #0
   jp       z, fin

   ld       a, (estado_partida)
   cp       #22
   jp       z, finniveles

   ld       a, (estado_partida_ant)
   ld       b, a
   ld       a, (estado_partida)
   sub      b
   cp       #0
   jp       nz, next

   call     man_restart_init
   call     man_restart_run
   jp       back

   fin:
   call     man_finjuego_init
   call     man_finjuego_run
   jp new_game
   

   next:
   call     man_next_level_init
   call     man_next_level_run
   ld       a,    (estado_partida)
   ld       (estado_partida_ant), a
   jp       back

   finniveles:
   ;call     man_next_level_init
   ;call     man_next_level_run
   jp       new_game

   ret